/**
 * Copyright (c) 2012, Martin Foster <martin@ethereal-realms.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in the 
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Ethereal-Realms.org nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lyra.Host;

/**
 * Event that is triggered when we have lost a connection.  Loosing a connection
 * necessitates telling any other instance that their connection may be
 * affected as well.
 * <p>
 * @author Martin Foster
 * @version 1.0.5, %Revision, 174%, %LastChangedDate, 27/10/12 3:33PM%
 * @see org.lyra.Host.LossListener
 */
public class LossEvent {
	
	// Source element
	private Object Source;
	
	// Time stamp
	@SuppressWarnings("unused")
	private Long Timestamp;
	
	// Exception error
	private Exception Error;
	
	/**
	 * Primary constructor for the event generated.   Takes the source of the
	 * caller and all of the updates that have happened at server level.
	 * <p>
	 * @param source Source object triggering the event.
	 * @param error Exception generated during error.
	 */
	public LossEvent(Object source, Exception error) {
		
		// Pass up elements for later use
		Source    = source;
		Error	= error;
		
		// Calculate the time
		Timestamp = System.currentTimeMillis();
	}
	// End of Constructor Class
	
	
	/**
	 * The object on which the event initially occurred. 
	 * <p>
	 * @return The object on which the Event initially occurred.
	 */
	public Object getSource() {
		
		// Straight forward passage
		return Source;
	}
	// End of getSource Method

	
	/**
	 * The Exception on which the event initially occurred. 
	 * <p>
	 * @return The exception which triggered this event.
	 */
	public Object getException() {
		
		// Straight forward passage
		return Error;
	}
	// End of getSource Method

}
