/**
 * Copyright (c) 2012, Martin Foster <martin@ethereal-realms.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in the 
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Ethereal-Realms.org nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.lyra.Interface.Chart;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.lyra.Lyra;
import org.lyra.Interface.Constants;
import org.lyra.Listener.WindowSizeListener;
import org.lyra.Widgets.RegexConstrainedDocument;
import org.lyra.Widgets.ResizeIcon;

/**
 * This class deals with all of the user interface elements necessary to
 * save a chart to a file for later use. When complete will pass on a 
 * Map[String,String] element with all the necessary details.
 * <p>
 * @author Martin Foster
 * @version 1.0.5, %Revision, 174%, %LastChangedDate, 27/10/12 3:33PM%
 * @since 1.0.5
 */
 public class SaveDialog extends JDialog implements Runnable {
 	 
	/*
	 * Visual Elements
	 */
	
	// A Necessary Evil
	private static final long serialVersionUID = 1L;
	
	// Overall Holder
	private JPanel panelHolder = null;
	
	// Options and Features
	private JPanel panelOptions = null;
		
	private JLabel labelWidth = null;
	private JTextField fieldWidth = null;
		
	private JLabel labelHeight = null;
	private JTextField fieldHeight = null;
	
	// Format Section
	private JPanel panelFormat = null;
	private JLabel labelFormat = null;

	private JRadioButton radioJPEG = null;
	private JRadioButton radioPNG = null;
	private JRadioButton radioSVG = null;

	// File Handler
	private JPanel panelFile = null;
	private JLabel labelFile = null;
	private JTextField fieldFile = null;
	private JButton buttonFile = null;

	// Control Section
	private JPanel panelControl = null;
	private JButton buttonCancel = null;
	private JButton buttonSave = null; 
	
	// Status Section
	private JPanel panelStatus = null;
	private JLabel iconResize = null;
	
	/*
	 * Package Elements
	 */

	// Language Element
	private Map<String, String> Lang;
	
	
	/*
	 * Class Elements
	 */
	 
	// Thread element
	private Thread Waitor;
	
	// Process track
	private boolean Process;
	
	// Action handler
	private Interact Action;
	
	// Button group access
	private ButtonGroup Buttons = null;
	
	// Details for the save
	private Map<String, String> Return = null;
	
	// Elements for storage
	private int Width;
	private int Height;
	
	// Positional Information
	private Window Parent;
	
	
 	/**
 	 * Primary constructor used by the application.  Will provide logging and
 	 * language independent display.
 	 * <p>
 	 * @param lang Language map.
 	 * @param width Current width of chart.
 	 * @param height Current height of chart.
 	 * @param parent Frame to block.
 	 */
 	public SaveDialog(Map<String,String> lang, int width, int height, Window parent) {
	
 		// Let the inherited class do its work
 		super(parent);
	
 		// Pass up
 		this.Lang   = lang;
 		this.Width  = width;
 		this.Height = height;
 		this.Parent = parent;
 		
 		
 		// Create Action Handler
 		this.Action = new Interact(this);
 		
 		// Establish Window Listener
 		this.addWindowListener(Action);
 		
 		// Initialise look and feel
 		initDialog(parent);
 	}
 	// End of Constructor Class
 	 
 	 
	/**
 	 * Element which will deal with the creation of the class instance and
 	 * extracting the information prior to returning it to the calling program.
 	 * <p>
 	 * @param lang Language map.
 	 * @param width Current width of chart.
 	 * @param height Current height of chart.
 	 * @param parent Frame to block.
 	 * @return Map containing the details for saving the chart.
 	 */
 	static public Map<String, String> getVerdict(Map<String,String> lang, int width, int height, Window parent) {

 	 	// Create a new instance and make visible
 	 	SaveDialog cd = new SaveDialog(lang, width, height, parent);

 	 	
 	 	// Create a new thread that will wait for a completed Host entry
 	 	cd.Waitor = new Thread(cd);
 	 	cd.Waitor.start();
	
 	 	// Set the display to be visible
 	 	cd.setVisible(true);
	
 	 	// Wait out for our needs
 	 	try {
			// Join in and wait for the results
			cd.Waitor.join();
		} 
	
		// Something went wrong so just pass on the defaults
		catch (InterruptedException e) { /* This space left intentionally blank */ }
	
		// Return element
		return cd.Return;
	}
	// End of getVerdict Method


	/**
	 * This method will wait out until the HashMap is defined.  If not it will
	 * sleep until interrupted.
	 */
	public void run() {
	
	  	/*
	  	 * We essentially wait until such a time that the user has opted to 
	  	 * do something.  Otherwise we have an arbitrarily large sleep period
	  	 * as to prevent unnecessary cycles.
	  	 */
		while (Process) 
		{
	  	   	try {
	  	   	   	// Nothing for us to do sleep
	  	   	   	Thread.sleep(300000);
	  	   	} 
		
	  	   	/*
	  	   	 * Another component has interrupted us from our sleep.  This
	  	   	 * should mean the end of the cycle.
	  	   	 */
	  	   	 catch (InterruptedException e) { /* This space left intentionally blank */ }
		}
	}
	// End of Run Method
	
	
	/**
	 * Establish the look and feel for the dialog box.
	 * <p>
	 * @param parent Window instance of the parent.
	 */
	private void initDialog(Window parent) {
		
		// Language Element
		this.setTitle(Lang.get("Graphs.Save.Window.Title"));
		
		// Adjust Icon
		this.setIconImage(new ImageIcon(
			getClass().getResource(
				Lyra.IMG_SOURCE + Lyra.LYRA)).getImage());
		
		// Identifier
		this.setName("SAVER");
		
		// Modal Behaviour
		this.setModal(true);

		// Initial Layout
		this.getContentPane().setLayout(new BorderLayout());
		
		// Insert Elements
		this.getContentPane().add(
			getPanelHolder(), 
			BorderLayout.CENTER
		  );
		this.getContentPane().add(
			getPanelStatus(), 
			BorderLayout.SOUTH
		  );
		
		// Pack the display
		this.pack();
		
		// Cannot make it smaller than the packed size
		this.setMinimumSize(
			this.getPreferredSize());
		
		// Centre
		this.setLocationRelativeTo(Parent);

	}
	// End of initDialog Method
	
	
	/**
	 * This method initialises a JPanel component providing row elements. 
	 * <p>
	 * @param label JLabel instance which will be resized.
	 * @param widget JComponent instance which will be added as-is.
	 * @return JPanel containing the elements properly established.
	 */
	 private JPanel getPanelRow(JLabel label, JComponent widget) {
	
	 	 /*
	 	  * In order to create a consistent look and feel with our use of 
		  * BoxLayout we will resize all the elements from this point.  However
		  * we maintain Height as to adjust if there are font changes.
		  */
		 label.setPreferredSize(
		 	 new Dimension(
		 	 	 Constants.getLabelWidth(),
		 	 	 label.getPreferredSize().height
		 	   )
		   );
	
		 /*
		  * Here we adjust the maximum values to be suitable for our needs.  
		  * Essentially we are not looking for text fields to grow exponentially
		  * when a user resizes itself.
		  */
		  widget.setMaximumSize(
		  	  new Dimension(
		  	  	  widget.getMaximumSize().width,
		  	  	  widget.getPreferredSize().height + Constants.getPaddingHeight()
		  	    )
		    );
	
		  /*
		   * We create the element at this point.  We essentially throw all
		   * of the instances into a newly created section and call it the
		   * end of a day.
		   */
	
		   // Create instance
		   JPanel row = new JPanel();

		   // Establish layout
		   row.setLayout(new BoxLayout(row, BoxLayout.X_AXIS));
	
		   // Insertion of the label and spacer
		   row.add(label);
		   row.add(Box.createHorizontalStrut(Constants.getLabelSpacer()));
	
		   // Insertion of the widget and end spacer
		   row.add(widget);
		   row.add(Box.createHorizontalGlue());
	
		   // Pass on row
		   return row;
	 }
	 // End of getPanelRow Method
	
	
	/**
	 * Create instance of panelOptions.
	 * <p>
	 * @return javax.swing.JPanel
	 */
	private JPanel getPanelOptions() {
		
		// Check for Existence
		if (panelOptions == null) {
			
			// Create Instance
			panelOptions = new JPanel();
			
			// Establish layout
			panelOptions.setLayout(
				new BoxLayout(getPanelOptions(), BoxLayout.Y_AXIS));
			
			// Look and Feel
			panelOptions.setBorder(
				BorderFactory.createTitledBorder(
					null, 
					Lang.get("Graphs.Save.Options"), 
					TitledBorder.DEFAULT_JUSTIFICATION, 
					TitledBorder.DEFAULT_POSITION, 
					new Font("Dialog", Font.BOLD, 12), 
					new Color(51, 51, 51)
				  )
			  );
			
			/*
			 * We make an active choice to match the look and feel
			 * of the rest of the application here.  Even though we
			 * could have created a slightly different look that was
			 * more segregated and perhaps efficient.
			 */
			 
			// Options for Width and Height
			panelOptions.add(getPanelRow(
				getLabelWidth(),
				getFieldWidth()), null
			  );
			panelOptions.add(getPanelRow(
				getLabelHeight(),
				getFieldHeight()), null
			  );
			
			// Spacer
			panelOptions.add(
				Box.createVerticalStrut(
					Constants.getSectionGap()));
			
			// Ability to select file format
			panelOptions.add(getPanelRow(
				getLabelFormat(),
				getPanelFormat()), null
			  );
			
			// Spacer
			panelOptions.add(
				Box.createVerticalStrut(
					Constants.getSectionGap()));
			
			// Filename and location to save
			panelOptions.add(getPanelRow(
				getLabelFile(),
				getPanelFile()), null
			  );
			
		}
		
		// Return Element
		return panelOptions;
	}
	// End of getPanelDimensions Method
	

	/**
	 * Create instance of panelHolder.
	 * <p>
	 * @return javax.swing.JPanel
	 */
	private JPanel getPanelHolder() {
		
		// Check For Existence
		if (panelHolder == null) {
			
			// Create Instance
			panelHolder = new JPanel();
			
			// Establish Layout
			panelHolder.setLayout(
				new BoxLayout(getPanelHolder(), BoxLayout.Y_AXIS));
			
			// Insert Elements
			panelHolder.add(getPanelOptions(), null);
			panelHolder.add(getPanelControl(), null);
		}
		
		// Return Instance
		return panelHolder;
	}
	// End of getPanelHolder Method
	
	
	/**
	 * Create instance of labelWidth.
	 * <p>
	 * @return javax.swing.JLabel
	 */
	private JLabel getLabelWidth() {
		
		// Check for Existence
		if (labelWidth == null) {
			
			// Create Instance
			labelWidth = new JLabel();
			
			// Establish Language
			labelWidth.setText(
				Lang.get("Graphs.Save.Graph.Width"));
		}
		
		// Return Instance
		return labelWidth;
		
	}
	// End of getLabelWidth
	
	
	/**
	 * Create instance of fieldWidth.
	 * <p>
	 * @return javax.swing.JTextField
	 */
	private JTextField getFieldWidth() {
		
		// Check for Existence
		if (fieldWidth == null) {
			
			// Create verification document
			RegexConstrainedDocument doc = new RegexConstrainedDocument("\\d{0,5}");
		
		
			// Create and establish baseline
			fieldWidth = new JTextField(doc, Integer.toString(Width), 5);

			// Identifier
			fieldWidth.setName("SAVE_WIDTH");
			
			// Adjust Sizing
			fieldWidth.setPreferredSize(
				new Dimension(
					fieldWidth.getPreferredSize().width + Constants.getPaddingWidth(),
					fieldWidth.getPreferredSize().height
				  )
			  );
		
			// Have maximum match
			fieldWidth.setMaximumSize(fieldWidth.getPreferredSize());
		}
		
		
		// Return Instance
		return fieldWidth;
	}
	// End of getFieldWidth
	
	
	/**
	 * Create instance of labelHeight.
	 * <p>
	 * @return javax.swing.JLabel
	 */
	private JLabel getLabelHeight() {
		
		// Check for Existence
		if (labelHeight == null) {
			
			// Create Instance
			labelHeight = new JLabel();
			
			// Establish Language
			labelHeight.setText(
				Lang.get("Graphs.Save.Graph.Height"));
		}
		
		// Return Instance
		return labelHeight;
		
	}
	// End of getLabelHeight
	
	
	/**
	 * Create instance of fieldHeight.
	 * <p>
	 * @return javax.swing.JTextField
	 */
	private JTextField getFieldHeight() {
		
		// Check for Existence
		if (fieldHeight == null) {
			
			// Create verification document
			RegexConstrainedDocument doc = new RegexConstrainedDocument("\\d{0,5}");
		
		
			// Create and establish baseline
			fieldHeight = new JTextField(doc, Integer.toString(Height), 5);

			// Identifier
			fieldHeight.setName("SAVE_HEIGHT");
			
			// Adjust Sizing
			fieldHeight.setPreferredSize(
				new Dimension(
					fieldHeight.getPreferredSize().width + Constants.getPaddingWidth(),
					fieldHeight.getPreferredSize().height
				  )
			  );
		
			// Have maximum match
			fieldHeight.setMaximumSize(fieldHeight.getPreferredSize());
		}
		
		
		// Return Instance
		return fieldHeight;
		
	}
	// End of getFieldHeight
	
	
	/**
	 * Create instance of panelFormat.
	 * <p>
	 * @return javax.swing.JPanel
	 */
	private JPanel getPanelFormat() {
		
		// Check For Existence
		if (panelFormat == null) {
			
			// Create Instance
			panelFormat = new JPanel();
			
			// Establish Layout
			panelFormat.setLayout(
				new BoxLayout(getPanelFormat(), BoxLayout.Y_AXIS));
			
			// Insert Elements
			panelFormat.add(getRadioJPEG(), null);
			panelFormat.add(getRadioPNG(), null);
			panelFormat.add(getRadioSVG(), null);
			
			
			// Establish Group
			Buttons = new ButtonGroup();
			
			// Insert into group
			Buttons.add(getRadioJPEG());
			Buttons.add(getRadioPNG());
			Buttons.add(getRadioSVG());

			// Default to one group
			getRadioJPEG().setSelected(true);
		}
		
		// Return Instance
		return panelFormat;
	}
	// End of getPanelFormat Method
	
	
	/**
	 * Create instance of labelFormat.
	 * <p>
	 * @return javax.swing.JLabel
	 */
	private JLabel getLabelFormat() {
		
		// Check for Existence
		if (labelFormat == null) {
			
			// Create Instance
			labelFormat  = new JLabel();

			// Language Element
			labelFormat.setText(
				Lang.get("Graphs.Save.Type"));
		}
		
		// Return Instance
		return labelFormat; 
		
	}
	// End of getlabelFormat
	
	
	/**
	 * Create instance of radioJPEG.
	 * <p>
	 * @return javax.swing.JRadioButton
	 */
	private JRadioButton getRadioJPEG() {
		
		// Check for Existence
		if (radioJPEG == null) {
			
			// Create Instance
			radioJPEG = new JRadioButton(
							Lang.get("Graphs.Save.Type.JPEG"));
			
			// Identifier
			radioJPEG.setName("JPEG");
		}
		
		// Return Instance
		return radioJPEG;
	}
	// End of getRadioJPEG Method
	

	/**
	 * Create instance of radioPNG.
	 * <p>
	 * @return javax.swing.JRadioButton
	 */
	private JRadioButton getRadioPNG() {
		
		// Check for Existence
		if (radioPNG == null) {
			
			// Create Instance
			radioPNG = new JRadioButton(
							Lang.get("Graphs.Save.Type.PNG"));
			
			// Identifier
			radioPNG.setName("PNG");
		}
		
		// Return Instance
		return radioPNG;
	}
	// End of getRadioPNG Method	
	
	
	/**
	 * Create instance of radioSVG.
	 * <p>
	 * @return javax.swing.JRadioButton
	 */
	private JRadioButton getRadioSVG() {
		
		// Check for Existence
		if (radioSVG == null) {
			
			// Create Instance
			radioSVG = new JRadioButton(
							Lang.get("Graphs.Save.Type.SVG"));
			
			// Identifier
			radioSVG.setName("SVG");
		}
		
		// Return Instance
		return radioSVG;
	}
	// End of getRadioSVG Method
	
	
	/**
	 * Create instance of panelFile.
	 * <p>
	 * @return javax.swing.JPanel
	 */
	private JPanel getPanelFile() {
		
		// Check For Existence
		if (panelFile == null) {
			
			// Create Instance
			panelFile = new JPanel();
			
			// Establish Layout
			panelFile.setLayout(
				new BoxLayout(getPanelFile(), BoxLayout.X_AXIS));
							
			// Insert Element
			panelFile.add(getFieldFile(), null);
			panelFile.add(getButtonFile(), null);
		}
		
		// Return Instance
		return panelFile;
	}
	// End of getPanelFile Method
	
	
	/**
	 * Create instance of labelFile.
	 * <p>
	 * @return javax.swing.JLabel
	 */
	private JLabel getLabelFile() {
		
		// Check for Existence
		if (labelFile == null) {
			
			// Create Instance
			labelFile  = new JLabel();

			// Language Element
			labelFile.setText(
				Lang.get("Graphs.Save.File"));
		}
		
		// Return Instance
		return labelFile; 
		
	}
	// End of getlabelFile
	
	
	/**
	 * Create instance of fieldFile.
	 * <p>
	 * @return javax.swing.JTextField
	 */
	private JTextField getFieldFile() {
		
		// Check for Existence
		if (fieldFile == null) {

			// Create and establish baseline
			fieldFile = new JTextField("", 25); 
			
			// This field cannot be edited
			fieldFile.setEditable(false);
			
			// Identifier
			fieldFile.setName("SAVE");
		}
		
		// Return Element
		return fieldFile;
	}
	// End of getFieldFile Method
	
	
	/**
	 * Create instance of buttonFile.
	 * <p>
	 * @return javax.swing.JButton
	 */
	private JButton getButtonFile() {
		
		// Check for Existence
		if (buttonFile == null) {
			
			// Create Instance
			buttonFile = new JButton();
			
			// Set Visual Elements
			buttonFile.setIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("File-Open"))));
			buttonFile.setToolTipText(Lang.get("Graphs.Save.File.Tooltip"));
			
			// Identifier and Listener
			buttonFile.setName("FILE");
			buttonFile.addActionListener(Action);
		}
		
		// Return Instance
		return buttonFile;
	}
	// End of getFieldButton Method
	
	
	/**
	 * Create instance of panelControl.
	 * <p>
	 * @return javax.swing.JPanel
	 */
	private JPanel getPanelControl() {
		
		// Check For Existence
		if (panelControl == null) {
			
			// Create Instance
			panelControl = new JPanel();
			
			// Establish Layout
			panelControl.setLayout(
				new BoxLayout(getPanelControl(), BoxLayout.X_AXIS));
			
			// Insert Elements
			panelControl.add(getButtonSave(), null);
			panelControl.add(getButtonCancel(), null);
			panelControl.add(Box.createHorizontalGlue(), null);
		}
		
		// Return Instance
		return panelControl;
	}
	// End of getPanelControl Method
	
	
	/**
	 * Create instance of buttonSave.
	 * <p>
	 * @return javax.swing.JButton
	 */
	private JButton getButtonSave() {
		
		// Check for Existence
		if (buttonSave == null) {
			
			// Create Instance
			buttonSave = new JButton();
			
			// Set Visual Elements
			buttonSave.setIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Button-Save"))));
			buttonSave.setToolTipText(Lang.get("Graph.Save.Button.Save"));
			buttonSave.setText(Lang.get("Button.Save"));
			
			// Until we get necessary detail this is disabled
			buttonSave.setEnabled(false);
			
			// Identifier and Listener
			buttonSave.setName("SAVE");
			buttonSave.addActionListener(Action);
		}
		
		// Return Instance
		return buttonSave;
	}
	// End of getSaveButton Method
	
	
	/**
	 * Create instance of buttonCancel.
	 * <p>
	 * @return javax.swing.JButton
	 */
	private JButton getButtonCancel() {
		
		// Check for Existence
		if (buttonCancel == null) {
			
			// Create Instance
			buttonCancel = new JButton();
			
			// Set Visual Elements
			buttonCancel.setIcon(new ImageIcon(
				getClass().getResource(
					Lyra.getIconByMap("Button-Cancel"))));
			buttonCancel.setToolTipText(Lang.get("Host.Save.Button.Cancel"));
			buttonCancel.setText(Lang.get("Button.Cancel"));
			
			// Identifier and Listener
			buttonCancel.setName("CANCEL");
			buttonCancel.addActionListener(Action);
		}
		
		// Return Instance
		return buttonCancel;
	}
	// End of getCancelButton Method
	
	
	/**
	 * This method initialises iconResize
	 * <p>
	 * @return javax.swing.JLabel
	 */
	private JLabel getIconResize() {
		
		// Verify existence of element
		if (iconResize == null) {

			// Create and establish baseline
			iconResize = new JLabel(new ResizeIcon());
			iconResize.setOpaque(false);
			
			/*
			 * Making this icon do something of use is one of the core reasons
			 * why it was added to the display itself.  Here we create an 
			 * instance and add the listener, since it is not used anywhere else
			 * we don't worry out creating an instance earlier.
			 */
			WindowSizeListener sizer = new WindowSizeListener(this);
			
			// Add in listener elements
			iconResize.addMouseListener(sizer);
			iconResize.addMouseMotionListener(sizer);
		}
		
		// Return label
		return iconResize;
	}
	// End of getIconResize Method
	
	
	/**
	 * This method initialises panelStatus	
	 * <p>
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPanelStatus() {
		
		// Determine existence
		if (panelStatus == null) {

			// Create instance
			panelStatus = new JPanel();
			
			// Establish layout
			panelStatus.setLayout(new BoxLayout(getPanelStatus(), BoxLayout.X_AXIS));
			
			// Add in resizing label/icon
			panelStatus.add(Box.createHorizontalGlue());
			panelStatus.add(getIconResize(), null);
		}
		
		// Return instance
		return panelStatus;
	}
	// End of getPanelStatus
	
	
    /**
     * A class dedicated to the action handling for the org.lyra.Host.Create'
     * class.  Since there is no real code re-use beyond the implementation it
     * was placed as a sub-class for code segregation.
     * <p>
     * @author Martin Foster
     *
     */
    private class Interact implements ActionListener, WindowListener {

    	/*
    	 * VARIABLES
    	 */
    	private JDialog Caller;

    	/**
    	 * Base constructor which provides the calling Class ability to grant
    	 * access to its windowing capability.  Such as window size.
    	 * <p>
    	 * @param caller JDialog instance.
    	 */
    	public Interact(JDialog caller) {
    		
    		// Pass up
    		this.Caller = caller;
    	}
    	// End of Constructor Class
    	
    	
    	/*
    	 * FUNCTIONS
    	 */
    	 
    	 /**
    	  * Will look at a specific ButtonGroup and determine exactly which one
    	  * is selected.
    	  * <p>
    	  * @param group ButtonGroup instance to compare.
    	  * @return String containing selected item's name.
    	  */
    	 private String getSelected(ButtonGroup group) {
    	 	 
    	 	 /*
		  * Radio Buttons are actually problematic in that they do
		  * not actively tell you which is selected.  So we need to
		  * search through and compare needlessly.
		  */
		 for (Enumeration<AbstractButton> e=group.getElements(); e.hasMoreElements();) {
						
		 	 // Extract Instance
		 	 JRadioButton b = (JRadioButton)e.nextElement();

		 	 // Compare against selected model
		 	 if (b.getModel() == Buttons.getSelection()) 
		 	 	 return b.getName();
		 }
		 
		 // Nothing to return if we make it this far
		 return null;
    	 }
    	 // End of getSelected Method

    	
    	/*
    	 * ACTION BASED EVENTS
    	 */
    	
    	
		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent evt) {
			
			/*
			 * A quick safety check to make sure we are dealing with the
			 * appropriate type.  When that has been confirmed we will extract
			 * the name and use that to determine appropriate action.
			 */
			if (evt.getSource() instanceof JComponent) {
				
				// Cast to a JButton Instance
				String ident = ((JComponent)evt.getSource()).getName();
				
				// Determine what was pressed
				// User has opted to Save
				if (ident.equals(getButtonSave().getName())) {
					
					// Need a proper return type
					Return = new HashMap<String,String>();
					
					// Capture file type
					Return.put("Type", getSelected(Buttons));
					
					// Insert additional element
					Return.put("Width",  getFieldWidth().getText());
					Return.put("Height", getFieldHeight().getText());
					Return.put("File",   getFieldFile().getText());
						
					// End Processing
					Process = false;
					
					// Hide and dispose of window
					Caller.setVisible(false);
					Caller.dispose();
					
					// Interrupt thread
					Waitor.interrupt();
				}
			
				// User wishes to select a daemon
				else if (ident.equals(getButtonFile().getName())) {
					
					/*
					 * Before we begin there is a requirement to filter out
					 * the extensions based on our selection.   So we
					 * extract that and create a display filter.
					 */
					 
					// Extract extensions
					String ext =  getSelected(Buttons);
					 
					// Create a new filter appropriate for selection
					FileNameExtensionFilter filter = new FileNameExtensionFilter(
							Lang.get("Graphs.Chooser.Description." + ext),
							Lang.get("Graphs.Chooser.Extensions." + ext)
							    .split(";")
						);

					
					// Create Instances
					JFileChooser fc = new JFileChooser();
					
					// We only want to select files of selected type
					fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
					fc.addChoosableFileFilter(filter);
					
					/*
					 * We only care if the user has decided to choose a file.
					 * Otherwise we ignore it completely.
					 */
					if (fc.showSaveDialog(Parent) == JFileChooser.APPROVE_OPTION) {
						
						// Get the source file
						File source = fc.getSelectedFile();
						
						// We test, if it fails assume extension missing
						String file = filter.accept(source)
							? source.getAbsolutePath()
							: source.getAbsolutePath() + "." + ext.toLowerCase();
						
						// Update field accordingly
						getFieldFile().setText(file);
						
						// Now we can press the save button accordingly
						getButtonSave().setEnabled(true);
					}
				}
				
				
				// User has opted to cancel
				else {
					
					/*
					 * All we do is pass on the necessary HashMap even though
					 * it is empty.  That will be the check to be done once it
					 * is returned.
					 */
					Return = new HashMap<String,String>();
					
					// End Processing
					Process = false;
					
					// Hide and dispose of window
					Caller.setVisible(false);
					Caller.dispose();
					
					// Interrupt thread
					Waitor.interrupt();
				}
			}
		}
		// End of actionPerformed Method
		
    	
    	/*
    	 * WINDOW BASED EVENTS
    	 */
		
		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
		 */
		public void windowClosing(WindowEvent evt) {
			
			/*
			 * There are plenty of elements that have to be confirmed when
			 * someone ignores the buttons and just closes the dialog.  So what
			 * we do is make sure that a proper state can be returned.
			 */	
			if (Return == null) {
				
				/*
				 * All we do is pass on the necessary HashMap even though
				 * it is empty.  That will be the check to be done once it
				 * is returned.
				 */
				Return = new HashMap<String,String>();
				
				// End Processing
				Process = false;
				
				// Hide and dispose of window
				Caller.setVisible(false);
				Caller.dispose();
				
				// Interrupt thread
				Waitor.interrupt();
			}
		}
		// End of windowClosing Method

		
		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
		 */
		public void windowClosed(WindowEvent evt) { /* This space intentionally left blank */ }
		
    	
		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
		 */
		public void windowActivated(WindowEvent evt) { /* This space intentionally left blank */ }


		
		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
		 */
		public void windowDeactivated(WindowEvent evt) { /* This space intentionally left blank */ }


		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
		 */
		public void windowDeiconified(WindowEvent evt) { /* This space intentionally left blank */ }


		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
		 */
		public void windowIconified(WindowEvent evt) { /* This space intentionally left blank */ }


		/* (non-Javadoc)
		 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
		 */
		public void windowOpened(WindowEvent evt) { /* This space intentionally left blank */ }
    }
    // End of Action Class
    
 }
 // End of SaveChartDialog
